#region Using declarations
using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.Drawing.Drawing2D;
using System.Xml.Serialization;
using System.Runtime.InteropServices;
using System.Text;
using System.Collections;
using System.Windows.Forms;
using NinjaTrader.Data;
using NinjaTrader.Gui.Chart;
using NinjaTrader.Gui.Design;
#endregion

namespace NinjaTrader.Indicator
{
    [Description("TheForce")]
    public class TheForce : Indicator
    {
        #region Variables
		private double open;
		private double close;
		private double high;
		private double low;
		private double prevBarClose = 0;
		private IDataSeries stochD;
		private IDataSeries stochK;		
		private IDataSeries macd;
		private IDataSeries emaOpen;		
		private IDataSeries emaClose;	
		private bool basicsellready = false;
		private bool basicbuyready = false;
		private bool advsellready = false;
		private bool advbuyready = false;
		private int basicbuyTriggerBar = 0;
		private int basicsellTriggerBar = 0;
		private int advbuyTriggerBar = 0;
		private int advsellTriggerBar = 0;
		private int emaPeriod = 5;
		private int stochDPeriod = 3;
		private int stochKPeriod = 5;
		private int stochSmooth = 3;
		private int macdFast = 12;
		private int macdSlow = 26;
		private int macdSmooth = 1;
		private string prevSignal = null;
		#endregion

        protected override void Initialize()
        {			
			Overlay				= true;
			DrawOnPricePanel    = true; 
            CalculateOnBarClose = false;
			PaintPriceMarkers = false;
        }

        protected override void OnBarUpdate()
        {
			if (CurrentBar < Math.Max(macdSlow,Math.Max(emaPeriod,stochKPeriod))) return;
			
			
			
			
			open = RoundPrice(Open[0]);
			close = RoundPrice(Close[0]);
			high = RoundPrice(High[0]);
			low = RoundPrice(Low[0]);
			prevBarClose = RoundPrice(Close[1]);
			

			stochD = (TheForceStochastics(stochDPeriod,stochKPeriod,stochSmooth).D);
			stochK = (TheForceStochastics(stochDPeriod,stochKPeriod,stochSmooth).K);
			macd = TheForceMACD(macdFast,macdSlow,macdSmooth).Macd;
			emaOpen = TheForceEMA(emaPeriod).EmaOpen;
			emaClose = TheForceEMA(emaPeriod).EmaClose;
			
		/*	if (stochK[0] < 20)// && prevSignal == "sell" || prevSignal == null) 
			{
				basicbuyready = true; 
				basicsellready = false;
				advbuyready = true; 
				advsellready = false;
			}
			if (stochK[0] > 80)// && prevSignal == "buy" || prevSignal == null) 
			{
				basicsellready = true; 
				basicbuyready = false;
				advsellready = true; 
				advbuyready = false;
			}
		*/	
			// I commented the basic signals because they are actually included in the advanced signals.
		/*	if (basicbuyready &&
				macd[0] > macd[1] &&
				stochK[0] > stochD[0] && 
				stochK[0] < 80 &&
				stochK[0] > 20 &&
				close > open &&
				low - close != 0 && //this prevents long signals when the close is at the bottom of the bar
				emaClose[0] > emaOpen[0] 
				)
			{
				prevSignal = "buy";
				basicbuyready = false;
				basicbuyTriggerBar = CurrentBar;
				DrawTriangleUp("Up"+CurrentBar,0,low-(2*TickSize),Color.Lime);
			}
			else if (!basicbuyready &&
				basicbuyTriggerBar == CurrentBar &&
				macd[0] > macd[1] &&
				stochK[0] > stochD[0] && 
				stochK[0] < 80 &&
				stochK[0] > 20 &&
				close > open &&
				low - close != 0 && //this prevents long signals when the close is at the bottom of the bar
				emaClose[0] > emaOpen[0]
				)
			{
				basicbuyready = false;
				DrawTriangleUp("Up"+CurrentBar,0,low-(2*TickSize),Color.Lime);
			}
			else if (basicsellready &&
				stochK[0] < stochD[0] && 
				stochK[0] < 80 &&
				stochK[0] > 20 &&
				macd[0] < macd[1] &&
				close < open &&
				high - close != 0 && //this prevents short signals when the close is at the top of the bar
				emaClose[0] < emaOpen[0] 
				)
			{
				prevSignal = "sell";
				basicsellready = false;
				basicsellTriggerBar = CurrentBar;
				DrawTriangleDown("Down"+CurrentBar,0,high+(2*TickSize),Color.Red);
			}
			else if (!basicsellready &&
				basicsellTriggerBar == CurrentBar &&
				stochK[0] < stochD[0] && 
				stochK[0] < 80 &&
				stochK[0] > 20 &&
				macd[0] < macd[1] &&
				close < open &&
				high - close != 0 && //this prevents short signals when the close is at the top of the bar
				emaClose[0] < emaOpen[0] 
				)
			{
				basicsellready = false;
				DrawTriangleDown("Down"+CurrentBar,0,high+(2*TickSize),Color.Red);
			}
			*/

			if (//!advbuyready &&
				macd[0] > macd[1] &&
				stochK[0] > stochD[0] && 
				stochK[0] < 80 &&
				close > open &&
				low - close != 0 && //this prevents long signals when the close is at the bottom of the bar
				emaClose[0] > emaOpen[0] 
				)
			{
				prevSignal = "buy";
				advbuyready = true;
				advsellready = false;
				advbuyTriggerBar = CurrentBar;
				DrawTriangleUp("Up"+CurrentBar,0,low-(2*TickSize),Color.Lime);
			}
		/*	else if (advbuyready &&
				advbuyTriggerBar == CurrentBar &&
				macd[0] > macd[1] &&
				stochK[0] > stochD[0] && 
				stochK[0] < 80 &&
				close > open &&
				low - close != 0 && //this prevents long signals when the close is at the bottom of the bar
				emaClose[0] > emaOpen[0] 
				)
			{
				prevSignal = "buy";
				advbuyready = true;
				DrawTriangleUp("Up"+CurrentBar,0,low-(2*TickSize),Color.Lime);
			}*/
			else 
			{
				prevSignal = null;
				advbuyready = false;
				RemoveDrawObject("Up"+CurrentBar);
			}
			
			if (//!advsellready &&
				stochK[0] < stochD[0] && 
				stochK[0] > 20 &&
				macd[0] < macd[1] &&
				close < open &&
				high - close != 0 && //this prevents short signals when the close is at the top of the bar
				emaClose[0] < emaOpen[0] 
				)
			{
				prevSignal = "sell";
				advsellready = true;
				advbuyready = false;
				advsellTriggerBar = CurrentBar;
				DrawTriangleDown("Down"+CurrentBar,0,high+(2*TickSize),Color.Red);
			}
		/*	else if (!advsellready &&
				advsellTriggerBar == CurrentBar &&
				stochK[0] < stochD[0] && 
				stochK[0] > 20 &&
				macd[0] < macd[1] &&
				close < open &&
				high - close != 0 && //this prevents short signals when the close is at the top of the bar
				emaClose[0] < emaOpen[0] 
				)
			{
				prevSignal = "sell";
				advsellready = false;
				DrawTriangleDown("Down"+CurrentBar,0,high+(2*TickSize),Color.Red);
			}*/
			else 
			{
				prevSignal = null;
				advsellready = false;
				RemoveDrawObject("Down"+CurrentBar);
			}
			
        }

        private string FormatPrice(double value)
        {
            return Bars.Instrument.MasterInstrument.FormatPrice(value);
        }

        private double RoundPrice(double value)
        {
            return Bars.Instrument.MasterInstrument.Round2TickSize(value);
        }

		#region Properties
		[Category("Parameters")]
		public int EmaPeriod
		{
			get { return emaPeriod; }
			set { emaPeriod = value; }
		}
		[Category("Parameters")]
		public int StochDPeriod
		{
			get { return stochDPeriod; }
			set { stochDPeriod = value; }
		}
		[Category("Parameters")]
		public int StochKPeriod
		{
			get { return stochKPeriod; }
			set { stochKPeriod = value; }
		}
		[Category("Parameters")]
		public int StochSmooth
		{
			get { return stochSmooth; }
			set { stochSmooth = value; }
		}
		[Category("Parameters")]
		public int MacdSlow
		{
			get { return macdSlow; }
			set { macdSlow = value; }
		}
		[Category("Parameters")]
		public int MacdFast
		{
			get { return macdFast; }
			set { macdFast = value; }
		}
		[Category("Parameters")]
		public int MacdSmooth
		{
			get { return macdSmooth; }
			set { macdSmooth = value; }
		}
        #endregion
    }
}

#region NinjaScript generated code. Neither change nor remove.
// This namespace holds all indicators and is required. Do not change it.
namespace NinjaTrader.Indicator
{
    public partial class Indicator : IndicatorBase
    {
        private TheForce[] cacheTheForce = null;

        private static TheForce checkTheForce = new TheForce();

        /// <summary>
        /// TheForce
        /// </summary>
        /// <returns></returns>
        public TheForce TheForce(int emaPeriod, int macdFast, int macdSlow, int macdSmooth, int stochDPeriod, int stochKPeriod, int stochSmooth)
        {
            return TheForce(Input, emaPeriod, macdFast, macdSlow, macdSmooth, stochDPeriod, stochKPeriod, stochSmooth);
        }

        /// <summary>
        /// TheForce
        /// </summary>
        /// <returns></returns>
        public TheForce TheForce(Data.IDataSeries input, int emaPeriod, int macdFast, int macdSlow, int macdSmooth, int stochDPeriod, int stochKPeriod, int stochSmooth)
        {
            checkTheForce.EmaPeriod = emaPeriod;
            emaPeriod = checkTheForce.EmaPeriod;
            checkTheForce.MacdFast = macdFast;
            macdFast = checkTheForce.MacdFast;
            checkTheForce.MacdSlow = macdSlow;
            macdSlow = checkTheForce.MacdSlow;
            checkTheForce.MacdSmooth = macdSmooth;
            macdSmooth = checkTheForce.MacdSmooth;
            checkTheForce.StochDPeriod = stochDPeriod;
            stochDPeriod = checkTheForce.StochDPeriod;
            checkTheForce.StochKPeriod = stochKPeriod;
            stochKPeriod = checkTheForce.StochKPeriod;
            checkTheForce.StochSmooth = stochSmooth;
            stochSmooth = checkTheForce.StochSmooth;

            if (cacheTheForce != null)
                for (int idx = 0; idx < cacheTheForce.Length; idx++)
                    if (cacheTheForce[idx].EmaPeriod == emaPeriod && cacheTheForce[idx].MacdFast == macdFast && cacheTheForce[idx].MacdSlow == macdSlow && cacheTheForce[idx].MacdSmooth == macdSmooth && cacheTheForce[idx].StochDPeriod == stochDPeriod && cacheTheForce[idx].StochKPeriod == stochKPeriod && cacheTheForce[idx].StochSmooth == stochSmooth && cacheTheForce[idx].EqualsInput(input))
                        return cacheTheForce[idx];

            TheForce indicator = new TheForce();
            indicator.BarsRequired = BarsRequired;
            indicator.CalculateOnBarClose = CalculateOnBarClose;
            indicator.Input = input;
            indicator.EmaPeriod = emaPeriod;
            indicator.MacdFast = macdFast;
            indicator.MacdSlow = macdSlow;
            indicator.MacdSmooth = macdSmooth;
            indicator.StochDPeriod = stochDPeriod;
            indicator.StochKPeriod = stochKPeriod;
            indicator.StochSmooth = stochSmooth;
            indicator.SetUp();

            TheForce[] tmp = new TheForce[cacheTheForce == null ? 1 : cacheTheForce.Length + 1];
            if (cacheTheForce != null)
                cacheTheForce.CopyTo(tmp, 0);
            tmp[tmp.Length - 1] = indicator;
            cacheTheForce = tmp;
            Indicators.Add(indicator);

            return indicator;
        }

    }
}

// This namespace holds all market analyzer column definitions and is required. Do not change it.
namespace NinjaTrader.MarketAnalyzer
{
    public partial class Column : ColumnBase
    {
        /// <summary>
        /// TheForce
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TheForce TheForce(int emaPeriod, int macdFast, int macdSlow, int macdSmooth, int stochDPeriod, int stochKPeriod, int stochSmooth)
        {
            return _indicator.TheForce(Input, emaPeriod, macdFast, macdSlow, macdSmooth, stochDPeriod, stochKPeriod, stochSmooth);
        }

        /// <summary>
        /// TheForce
        /// </summary>
        /// <returns></returns>
        public Indicator.TheForce TheForce(Data.IDataSeries input, int emaPeriod, int macdFast, int macdSlow, int macdSmooth, int stochDPeriod, int stochKPeriod, int stochSmooth)
        {
            return _indicator.TheForce(input, emaPeriod, macdFast, macdSlow, macdSmooth, stochDPeriod, stochKPeriod, stochSmooth);
        }

    }
}

// This namespace holds all strategies and is required. Do not change it.
namespace NinjaTrader.Strategy
{
    public partial class Strategy : StrategyBase
    {
        /// <summary>
        /// TheForce
        /// </summary>
        /// <returns></returns>
        [Gui.Design.WizardCondition("Indicator")]
        public Indicator.TheForce TheForce(int emaPeriod, int macdFast, int macdSlow, int macdSmooth, int stochDPeriod, int stochKPeriod, int stochSmooth)
        {
            return _indicator.TheForce(Input, emaPeriod, macdFast, macdSlow, macdSmooth, stochDPeriod, stochKPeriod, stochSmooth);
        }

        /// <summary>
        /// TheForce
        /// </summary>
        /// <returns></returns>
        public Indicator.TheForce TheForce(Data.IDataSeries input, int emaPeriod, int macdFast, int macdSlow, int macdSmooth, int stochDPeriod, int stochKPeriod, int stochSmooth)
        {
            if (InInitialize && input == null)
                throw new ArgumentException("You only can access an indicator with the default input/bar series from within the 'Initialize()' method");

            return _indicator.TheForce(input, emaPeriod, macdFast, macdSlow, macdSmooth, stochDPeriod, stochKPeriod, stochSmooth);
        }

    }
}
#endregion
